# from purview_py import PurviewType, PurviewEntity, Connection
import pprint, requests


class PurviewController(object):
    """A controller for Purview"""

    def __init__(self):
        pass

    def create_new_types(self, conn, types):
        # Check manditory fields
        # format request
        # post data
        #check response code

        requestdata = {"entityDefs":[], "classificationDefs":[], "structDefs":[], "relationshipDefs":[]}
        pp = pprint.PrettyPrinter(indent=4)
        
        for t in types:
            type_dict = t.format_for_requests()
            type_dict.pop('guid', None)

            pp.pprint(type_dict)

            if type_dict["category"] == "ENTITY":
                requestdata["entityDefs"].append(type_dict)
            if type_dict["category"] == "CLASSIFICATION":
                requestdata["classificationDefs"].append(type_dict)
            if type_dict["category"] == "STRUCT":
                requestdata["structDefs"].append(type_dict)
            if type_dict["category"] == "RELATIONSHIP":
                requestdata["relationshipDefs"].append(type_dict)
            
        pp = pprint.PrettyPrinter(indent=4)
        pp.pprint(requestdata)

        # Format Request
        r = requests.post(f"{conn.purviewEndpoint}/catalog/api/atlas/v2/types/typedefs", headers=conn.headers, json=requestdata)
        print(r.json())
        if r.status_code != 200:
            return f"{r.status_code}: {r.json()}"
        else:
            return r.status_code


"""
OWLPy
~~~~~~~~~~~~~~~~~~~~~
An extension module to facilitate API Models and functionality.
:copyright: (c) 2019 Spydernaz
:license: MIT, see LICENSE for more details.
"""
from .Connection import PurviewConnection



from purview_py.controller.type.Attribute import PurviewAttribute, PurviewRelationshipAttribute
from datetime import datetime
import requests, json, uuid, pprint


class PurviewType(object):
    
    def __init__(self, category, name, superTypes, subTypes=[], guid=str(uuid.uuid4()), createdBy="purview_py", updatedBy="purview_py", createTime=datetime.now(), updateTime=datetime.now(), version=2, description="", typeVersion="1.0", options={}, lastModifiedTS=None, attributeDefs=[], relationshipAttributeDefs=[], serviceType=None, newType=False):
        self.guid = guid
        self.category = category
        self.createdBy = createdBy
        self.updatedBy = updatedBy
        self.createTime = createTime
        self.updateTime = updateTime
        self.version = version
        self.name = name
        self.description = description
        self.typeVersion = typeVersion
        self.serviceType = serviceType
        self.options = options
        self.lastModifiedTS = lastModifiedTS
        self.attributeDefs = attributeDefs
        self.superTypes = superTypes
        self.subTypes = subTypes
        self.relationshipAttributeDefs = relationshipAttributeDefs
        self.newType = newType

    @classmethod
    def getTypeByName(cls, conn, name):
        urlheaders = {"Content-Type": "application/json", "Authorization": f"Bearer {conn.auth.return_token()}"}
        response = requests.get(f"{conn.purviewEndpoint}/catalog/api/atlas/v2/types/typedef/name/{name}", headers=urlheaders)
        if response.status_code == 200:
            return cls.getClassByJSON(response.json())
        else:
            raise Exception(response.status_code, response.json())

    @classmethod
    def getTypeByGUID(cls, conn, guid):
        urlheaders = {"Content-Type": "application/json", "Authorization": f"Bearer {conn.auth.return_token()}"}
        response = requests.get(f"{conn.purviewEndpoint}/catalog/api/atlas/v2/types/typedef/guid/{guid}", headers=urlheaders)
        if response.status_code == 200:
            return cls.getClassByJSON(response.json())
        else:
            raise Exception(response.status_code, response.json())

    @classmethod
    def getClassByJSON(cls, apiresp):
        
        # print(apiresp)
        
        args = dict(apiresp)
        tmpAttributes = []
        tmpRelAttributes = []

        # pp = pprint.PrettyPrinter(indent=4)
        

        for attr in args["attributeDefs"]:
            # pp.pprint(attr)
            tmpAttributes.append(PurviewAttribute(**attr))
        for attr in args["relationshipAttributeDefs"]:
            tmpRelAttributes.append(PurviewRelationshipAttribute(**attr))

        args["attributeDefs"] = tmpAttributes
        args["relationshipAttributeDefs"] = tmpRelAttributes

        # pp = pprint.PrettyPrinter(indent=4)
        # pp.pprint(args)

        t = cls(**args, newType=False)
        return t

    def format_for_requests(self):

        attrs = []
        if len(self.attributeDefs) > 0:
            for a in self.attributeDefs:
                attrs.append(vars(a))
        rel_attrs = []
        if len(self.relationshipAttributeDefs) > 0:
            for a in self.relationshipAttributeDefs:
                rel_attrs.append(vars(a))
        new_type_request = vars(self)
        new_type_request["attributeDefs"] = attrs
        new_type_request["relationshipAttributeDefs"] = rel_attrs

        new_type_request.pop('newType', None)

        # pp = pprint.PrettyPrinter(indent=4)
        # pp.pprint(requestdata)

        return new_type_request

    def update_def(self):
        pass

        



class Attribute(object):
    def __init__(self):
        pass

class PurviewRelationshipAttribute(Attribute):

    def __init__(self, name, typeName, relationshipTypeName, cardinality, valuesMinCount, valuesMaxCount, constraints=[], isOptional=True, isUnique=False, isIndexable=False, includeInNotification=False, isLegacyAttribute=False):
        self.name = str(name)
        self.typeName = str(typeName)
        self.relationshipTypeName = str(relationshipTypeName)
        self.cardinality = str(cardinality)
        self.valuesMinCount = int(valuesMinCount)
        self.valuesMaxCount = int(valuesMaxCount)
        self.isOptional = bool(isOptional)
        self.isUnique = bool(isUnique)
        self.isIndexable = bool(isIndexable)
        self.includeInNotification = bool(includeInNotification)
        self.isLegacyAttribute = bool(isLegacyAttribute)

    # def get_attr_dict
    

class PurviewAttribute(Attribute):

    def __init__(self, name, typeName, cardinality, valuesMinCount, valuesMaxCount, isOptional=True, isUnique=False, isIndexable=False, includeInNotification=False, isLegacyAttribute=False):
        self.name = str(name)
        self.typeName = str(typeName)
        self.cardinality = str(cardinality)
        self.valuesMinCount = int(valuesMinCount)
        self.valuesMaxCount = int(valuesMaxCount)
        self.isOptional = bool(isOptional)
        self.isUnique = bool(isUnique)
        self.isIndexable = bool(isIndexable)
        self.includeInNotification = bool(includeInNotification)
        self.isLegacyAttribute = bool(isLegacyAttribute)




import requests


class TokenAuth(object):
    
    def __init__(self, CLIENT_ID, CLIENT_SECRET, TENANT_ID):
        self.CLIENT_ID = CLIENT_ID
        self.CLIENT_SECRET = CLIENT_SECRET
        self.TENANT_ID = TENANT_ID

        self.TOKEN_ENDPOINT = f"https://login.microsoftonline.com/{TENANT_ID}/oauth2/token"

        self.fetch_token()

    def fetch_token(self):
        urlheaders = {"Content-Type": "application/x-www-form-urlencoded"}
        urldata = {
            "grant_type": "client_credentials",
            "client_id": self.CLIENT_ID,
            "client_secret": self.CLIENT_SECRET,
            "resource": "https://purview.azure.net"
        }
        request = requests.post(self.TOKEN_ENDPOINT, headers=urlheaders, data=urldata, allow_redirects=True)
        
        if request.status_code == 200:
            self.TOKEN = request.json()["access_token"]
        else:
            raise Exception(request.status_code, request.text)
    
    def return_token(self):
        return self.TOKEN

    def refresh_token(self):
        # TODO
        pass




import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name = 'purview_py',
    version = '0.0.10',
    license='MIT',
    description = 'Python wrapper for MSFT Purview and Apache Atlas APIs',
    author = 'Nathaniel Vala',
    author_email = 'nathanielvala@hotmail.com',
    url = 'https://github.com/spydernaz/purview_py',
    keywords = ['API', 'Data Management', 'Purview', 'Apache', 'Atlas', 'Metadata'],
    packages=setuptools.find_packages(),
    install_requires=[
        'requests'
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    # package_dir={"": "purview_py"},
    # packages=setuptools.find_packages(where="purview_py"),
    python_requires=">=3.6"

)


"""
OWLPy
~~~~~~~~~~~~~~~~~~~~~
An extension module to facilitate API Models and functionality.
:copyright: (c) 2019 Spydernaz
:license: MIT, see LICENSE for more details.
"""
# name = "OWLPy"
from .auth.TokenAuth import TokenAuth
from .conn.Connection import PurviewConnection
from .controller.Controller import PurviewController
from .controller.type.PurviewType import PurviewType
from .controller.type.Attribute import PurviewAttribute, PurviewRelationshipAttribute




class PurviewConnection(object):
    
    def __init__(self, resource, auth):
        self.purviewEndpoint = f"https://{resource}.purview.azure.com"
        self.auth = auth
        self.headers = {"Content-Type": "application/json", "Authorization": f"Bearer {self.auth.return_token()}"}

